package es.guiguegon.gallerymodule.fragments;

import android.Manifest;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
import es.guiguegon.gallerymodule.GalleryActivity;
import es.guiguegon.gallerymodule.R;
import es.guiguegon.gallerymodule.adapters.GalleryAdapter;
import es.guiguegon.gallerymodule.helpers.CameraHelper;
import es.guiguegon.gallerymodule.helpers.GalleryHelper;
import es.guiguegon.gallerymodule.helpers.PermissionsManager;
import es.guiguegon.gallerymodule.model.GalleryMedia;
import es.guiguegon.gallerymodule.utils.ScreenUtils;
import java.util.ArrayList;
import java.util.List;

public class GalleryFragment extends Fragment
        implements GalleryAdapter.OnGalleryClickListener, GalleryHelper.GalleryHelperListener {

    private static final String KEY_GALLERY_MEDIAS = "key_gallery_medias";

    Toolbar toolbar;
    RecyclerView galleryRecyclerView;
    ProgressBar loadingProgressBar;
    Button btnRetry;
    TextView emptyTextview;

    ArrayList<GalleryMedia> galleryMedias = new ArrayList<>();

    GalleryAdapter galleryAdapter;
    StaggeredGridLayoutManager staggeredGridLayoutManager;
    CameraHelper cameraHelper;
    GalleryHelper galleryHelper;

    public static GalleryFragment newInstance() {
        GalleryFragment fragment = new GalleryFragment();
        Bundle arguments = new Bundle();
        fragment.setArguments(arguments);
        return fragment;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        galleryHelper = GalleryHelper.getInstance();
        cameraHelper = CameraHelper.getInstance();
        galleryHelper.onCreate(getContext(), this);
        cameraHelper.onCreate(getContext());
        return inflater.inflate(R.layout.fragment_gallery, container, false);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        galleryHelper.onDestroy();
        cameraHelper.onDestroy();
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        toolbar = (Toolbar) view.findViewById(R.id.toolbar);
        galleryRecyclerView = (RecyclerView) view.findViewById(R.id.gallery_recycler_view);
        loadingProgressBar = (ProgressBar) view.findViewById(R.id.loading_progress_bar);
        btnRetry = (Button) view.findViewById(R.id.btn_retry);
        emptyTextview = (TextView) view.findViewById(R.id.empty_textview);
        setupUi();
        if (savedInstanceState == null) {
            init();
        }
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        outState.putParcelableArrayList(KEY_GALLERY_MEDIAS, galleryMedias);
        super.onSaveInstanceState(outState);
    }

    @Override
    public void onViewStateRestored(@Nullable Bundle savedInstanceState) {
        super.onViewStateRestored(savedInstanceState);
        if (savedInstanceState != null) {
            galleryMedias = savedInstanceState.getParcelableArrayList(KEY_GALLERY_MEDIAS);
            afterConfigChange();
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        GalleryMedia galleryMedia = cameraHelper.onGetPictureIntentResults(requestCode, resultCode, data);
        if (galleryMedia != null) {
            onGalleryMedia(galleryMedia);
        }
    }

    protected void setupUi() {
        setToolbar(toolbar);
        int columns = getMaxColumns();
        galleryAdapter = new GalleryAdapter(getContext(), columns);
        staggeredGridLayoutManager = new StaggeredGridLayoutManager(columns, StaggeredGridLayoutManager.VERTICAL);
        galleryRecyclerView.setLayoutManager(staggeredGridLayoutManager);
        galleryRecyclerView.setAdapter(galleryAdapter);
        galleryAdapter.setOnGalleryClickListener(this);
        btnRetry.setOnClickListener(this::onButtonRetryClick);
    }

    protected void init() {
        getGalleryMedia();
    }

    protected void afterConfigChange() {
        fillGalleryMedia();
    }

    public int getMaxColumns() {
        int widthRecyclerViewMediaFiles = ScreenUtils.getScreenWidth(getContext());
        int sizeItemsRecyclerView = getResources().getDimensionPixelSize(R.dimen.gallery_item_min_width);
        return widthRecyclerViewMediaFiles / sizeItemsRecyclerView;
    }

    private void fillGalleryMedia() {
        if (!galleryMedias.isEmpty()) {
            galleryAdapter.addGalleryImage(galleryMedias);
            hideEmptyList();
        } else {
            showEmptyList();
        }
    }

    private void showLoading() {
        loadingProgressBar.setVisibility(View.VISIBLE);
    }

    private void hideLoading() {
        loadingProgressBar.setVisibility(View.GONE);
    }

    private void showEmptyList() {
        galleryRecyclerView.setVisibility(View.GONE);
        emptyTextview.setVisibility(View.VISIBLE);
    }

    private void hideEmptyList() {
        galleryRecyclerView.setVisibility(View.VISIBLE);
        emptyTextview.setVisibility(View.GONE);
        btnRetry.setVisibility(View.GONE);
    }

    public void showRetry() {
        btnRetry.setVisibility(View.VISIBLE);
    }

    protected void setToolbar(Toolbar toolbar) {
        ((AppCompatActivity) getActivity()).setSupportActionBar(toolbar);
        ((AppCompatActivity) getActivity()).getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        ((AppCompatActivity) getActivity()).getSupportActionBar().setHomeButtonEnabled(true);
        ((AppCompatActivity) getActivity()).getSupportActionBar().setTitle(R.string.gallery_toolbar_title);
    }

    @Override
    public void onGalleryClick(GalleryMedia galleryMedia) {
        onGalleryMediaSelected(galleryMedia);
    }

    @Override
    public void onCameraClick() {
        try {
            PermissionsManager.requestMultiplePermissions((ViewGroup) getView(), () -> camera(getActivity()),
                    Manifest.permission.CAMERA, Manifest.permission.WRITE_EXTERNAL_STORAGE);
        } catch (Exception e) {
            showError(getString(R.string.gallery_exception_necessary_permissions));
        }
    }

    public void getGalleryMedia() {
        try {
            PermissionsManager.requestMultiplePermissions((ViewGroup) getView(), this::getGalleryImages,
                    this::showRetry, Manifest.permission.WRITE_EXTERNAL_STORAGE);
        } catch (Exception e) {
            showError(getString(R.string.gallery_exception_necessary_permissions));
            showRetry();
        }
    }

    public void camera(Activity activity) {
        cameraHelper.dispatchGetPictureIntent(activity);
    }

    public void getGalleryImages() {
        galleryHelper.getGalleryAsync();
        showLoading();
    }

    public void showError(String message) {
        hideLoading();
        Toast.makeText(getContext(), message, Toast.LENGTH_LONG).show();
    }

    public void onGalleryMedia(GalleryMedia galleryMedia) {
        this.galleryMedias.add(0, galleryMedia);
        galleryAdapter.addGalleryImage(galleryMedia);
        hideEmptyList();
    }

    public void onGalleryMedia(List<GalleryMedia> galleryMedias) {
        this.galleryMedias.addAll(0, galleryMedias);
        galleryAdapter.addGalleryImage(galleryMedias);
        hideEmptyList();
    }

    public void onGalleryMediaSelected(GalleryMedia galleryMedia) {
        Intent intent = new Intent();
        intent.putExtra(GalleryActivity.RESULT_GALLERY_MEDIA, galleryMedia);
        getActivity().setResult(Activity.RESULT_OK, intent);
        getActivity().supportFinishAfterTransition();
    }

    void onButtonRetryClick(View view) {
        init();
    }

    @Override
    public void onGalleryReady(List<GalleryMedia> galleryMedias) {
        hideLoading();
        onGalleryMedia(galleryMedias);
    }

    @Override
    public void onGalleryError() {
        hideLoading();
        showError(getString(R.string.gallery_something_went_wrong));
        showRetry();
    }
}